import configparser
import tensorflow as tf
from mppi import MPPI
from model import Nagumo1D_boundary
from policy import Policy
import numpy as np

np_dtype = np.float32
tf_dtype = tf.float32

np.set_printoptions(linewidth=1000)
##################intialize random seed and pass it to functions###########################
seed = np.random.seed(1)
tfseed = tf.random.set_random_seed(1)
config = configparser.RawConfigParser()
config.read('1D_nagumo_boundary.cfg')

dt = config.getfloat('sec1', 'dt') #time increment
Tf = config.getfloat('sec1', 'Tf') #final time
a = config.getfloat('sec1', 'a') #length of heat line
J = config.getint('sec1', 'J') #number of noise expansion terms
iters = config.getint('sec1', 'iters') #number of MPC iterations per time step
rollouts = config.getint('sec1', 'rollouts') #number of rollouts
rho = config.getfloat('sec1', 'rho') #acts as "temperature", squared inverse of variance
learnrate = config.getfloat('sec1', 'learnrate') # learning rate for updates
epsilon = config.getfloat('sec1', 'epsilon') #heat pde constant (scaling of h_xx)
l2_scale = config.getfloat('sec1', 'l2_scale') #regularization scaling
desired_value = np.float32(config.getfloat('sec1', 'desired_value')) #goal temperature for MPC
Q = config.getfloat('sec1', 'Q') #a single value
load_check = config.getboolean('sec1', 'load_network_check')
load_file_name = config.get('sec2', 'load_file_name')
wave_speed = config.getfloat('sec1', 'wave_speed')

mpc_steps = int(np.ceil(Tf/dt)) # total number of MPC timesteps

# Get/compute actuator data
# actuator_locs = np.asarray(config.get('sec2', 'actuator_locs').split(','), dtype=np_dtype) * np.float32(a) #actuator locations
hidden_layers = np.asarray(config.get('sec2', 'hidden_layers').split(','), dtype=np.int32) #list of hidden nodes in each layer
# N = len(actuator_locs) #number of actuators
N = 1 # Suppression only task

class Params(object):
	def __init__(self):
		self.l2_scale = l2_scale
		self.dt = dt
		self.a = a
		self.bj = np.sqrt(2.0 * self.dt/self.a)
		self.iterations = iters #
		self.learnrate = learnrate
		self.J = J 
		self.avg_spacing = a/J
		self.epsilon = epsilon
		self.rho = rho #tuning parameter
		self.N = N
		self.mpc_steps = mpc_steps
		self.rollouts = rollouts #tuning parameter
		# self.actuator_locs = actuator_locs
		# self.sig_xx = sig_xx
		self.hidden_layers = hidden_layers
		self.Q = Q #tuning parameter

		self.desired_state = np.zeros((1, J+1), dtype=np_dtype)
		xvals = np.arange(0, a+self.avg_spacing, self.avg_spacing)
		hvals = 1/(1+np.exp(-(2-xvals)/np.sqrt(2)))
		self.h_0 = tf.constant(hvals, dtype=tf_dtype) * tf.ones((self.rollouts, self.J+1), dtype=tf_dtype)

		self.num_layers = len(hidden_layers)-1
		range1 = np.int32(np.arange(np.round(0.7*J), np.round(0.99*J)+1, 1) )
		self.desired_indices = range1
		self.desired_state[0, range1] = desired_value
		print("Desired state: ",self.desired_state)

		self.desired_value = desired_value

		self.num_weights = (J+1)*hidden_layers[0] + hidden_layers[0]
		for j in range(self.num_layers):
			self.num_weights += hidden_layers[j]*hidden_layers[j+1] + hidden_layers[j+1]
		self.num_weights += hidden_layers[-1]*N + N
		print("Constructing a network with ",self.num_weights,"parameters")
		self.random_seed = seed
		self.tf_random_seed = tfseed
		self.load_file_name = load_file_name
		self.load_check = load_check
		self.wave_speed = wave_speed

params = Params()
policy = Policy(params)
nagumo1d = Nagumo1D_boundary(params)
mppi = MPPI(params, nagumo1d, policy)
mppi._mppi()



		